@@ -89,7 +89,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 for concepto in conceptos_list:
                     conceptos.appendChild(self.get_conceptos(concepto))
                 dom.appendChild(conceptos)
-            if self.data.get('iva') != '0' and self.data.get('iva') != 0:
+            if len(self.impuestos_trasladados) > 0 or len(self.impuestos_retenidos) > 0:
                 dom.appendChild(self.get_impuestos())
         elif self.tipo_comprobante == PAGO:
             dom.setAttribute('SubTotal','0')
@@ -186,14 +186,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
     def get_impuestos(self):
         try:
             impuestos = self.dom.createElement('cfdi:Impuestos')
-            if len(self.impuestos_trasladados) > 0:
-                traslados = self.dom.createElement('cfdi:Traslados')
-                total_traslados = 0
-                for traslado in self.impuestos_trasladados:
-                    total_traslados += float(traslado.get('importe', 0))
-                    traslados.appendChild(self.get_traslados(traslado))
-                impuestos.appendChild(traslados)
-                impuestos.setAttribute('TotalImpuestosTrasladados', str(total_traslados))
+
             if len(self.impuestos_retenidos) > 0:
                 retenciones = self.dom.createElement('cfdi:Retenciones')
                 total_retenciones = 0
@@ -203,6 +196,15 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 impuestos.appendChild(retenciones)
                 impuestos.setAttribute('TotalImpuestosRetenidos', str(total_retenciones))
 
+            if len(self.impuestos_trasladados) > 0:
+                traslados = self.dom.createElement('cfdi:Traslados')
+                total_traslados = 0
+                for traslado in self.impuestos_trasladados:
+                    total_traslados += float(traslado.get('importe', 0))
+                    traslados.appendChild(self.get_traslados(traslado))
+                impuestos.appendChild(traslados)
+                impuestos.setAttribute('TotalImpuestosTrasladados', str(total_traslados))
+
             return impuestos
         except Exception as e:
             raise Exception("Ocurrio un error. Error::: " + str(e))
@@ -228,16 +230,16 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
         try:
             complemento = self.dom.createElement('cfdi:Complemento')
             if self.complemento == COMPLEMENTO_DONATARIAS:
-                complemento.appendChild(self.get_donatarias())
+                complemento.appendChild(self.get_complemento_donatarias())
             elif self.complemento == COMPLEMENTO_PAGOS:
-                complemento.appendChild(self.get_pagos())
+                complemento.appendChild(self.get_complemento_pagos())
             elif self.complemento == COMPLEMENTO_CARTA_PORTE:
                 complemento.appendChild(self.get_complemento_cartaporte())
             return complemento
         except Exception as e:
             raise Exception("Ocurrio un error. Error::: " + str(e))
 
-    def get_donatarias(self):
+    def get_complemento_donatarias(self):
         try:
             donataria = self.dom.createElement('donat:Donatarias')
             donataria.setAttribute('xmlns:donat','http://www.sat.gob.mx/donat')
@@ -254,7 +256,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
         except Exception as e:
             raise Exception("Ocurrio un error. Error::: " + str(e))
 
-    def get_pagos(self):
+    def get_complemento_pagos(self):
         try:
             pagos = self.dom.createElement('pago10:Pagos')
             pagos.setAttribute('Version','1.0')
@@ -329,7 +331,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 raise Exception('Se debe especificar una descripción para poder timbrar y posteriormente crear una poliza.')
             cfdi_concepto.setAttribute('ValorUnitario', str(concepto.get('valor_unitario')))
             cfdi_concepto.setAttribute('Importe', str(concepto.get('importe')))
-            if self.complemento not in [COMPLEMENTO_CARTA_PORTE, COMPLEMENTO_DONATARIAS]:
+            if self.tipo_comprobante != TRASLADO:
                 #revisar en el manual si lleva impuestos este tipo de comprobante
                 cfdi_concepto.appendChild(self.get_impuestos_concepto(concepto))
             if concepto.get('informacion_aduanera'):
@@ -370,8 +372,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
         retencion.setAttribute('Impuesto', retencion_data.get('impuesto'))
         retencion.setAttribute('TipoFactor', retencion_data.get('tipo_factor', 'Tasa'))  # Tipo factor Tasa ó Cuota
         if retencion_data.get('tipo_factor') != 'Exento':
-            retencion.setAttribute('TasaOCuota',
-                                   str(retencion_data.get('tasa_cuota')))  # valor del impuesto que se envia
+            retencion.setAttribute('TasaOCuota', str(retencion_data.get('tasa_cuota')))  # valor del impuesto
             retencion.setAttribute('Importe', str(retencion_data.get('importe')))
         return retencion
 
@@ -428,28 +429,30 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 cartaporte_ubicacion = self.dom.createElement('cartaporte20:Ubicacion')
                 destination_counter = 1
                 if ubicacion_data.get('type') == 'origen':
-                    #cartaporte_ubicacion_origen = self.dom.createElement('cartaporte20:Origen')
+                    # cartaporte_ubicacion_origen = self.dom.createElement('cartaporte20:Origen')
                     cartaporte_ubicacion.setAttribute('IDUbicacion', 'OR{}'.format('000001'))
                     cartaporte_ubicacion.setAttribute('TipoUbicacion', 'Origen')
                     cartaporte_ubicacion.setAttribute('RFCRemitenteDestinatario', ubicacion_data.get('RFCRemitente'))
                     # cartaporte_ubicacion.setAttribute('NombreRFC', ubicacion_data.get('NombreRemitente'))
                     cartaporte_ubicacion.setAttribute('FechaHoraSalidaLlegada', ubicacion_data.get('FechaHoraSalida'))
-                    #cartaporte_ubicacion_origen.setAttribute('RFCRemitente', ubicacion_data.get('RFCRemitente'))
-                    #cartaporte_ubicacion_origen.setAttribute('NombreRemitente', ubicacion_data.get('NombreRemitente'))
-                    #cartaporte_ubicacion_origen.setAttribute('FechaHoraSalida', ubicacion_data.get('FechaHoraSalida'))
-                    #cartaporte_ubicacion.appendChild(cartaporte_ubicacion_origen)
+                    # cartaporte_ubicacion_origen.setAttribute('RFCRemitente', ubicacion_data.get('RFCRemitente'))
+                    # cartaporte_ubicacion_origen.setAttribute('NombreRemitente', ubicacion_data.get('NombreRemitente'))
+                    # cartaporte_ubicacion_origen.setAttribute('FechaHoraSalida', ubicacion_data.get('FechaHoraSalida'))
+                    # cartaporte_ubicacion.appendChild(cartaporte_ubicacion_origen)
                 elif ubicacion_data.get('type') == 'destino':
                     cartaporte_ubicacion.setAttribute('IDUbicacion', 'DE00000{}'.format(destination_counter))
-                    cartaporte_ubicacion.setAttribute('DistanciaRecorrida', str(ubicacion_data.get('DistanciaRecorrida')))
+                    cartaporte_ubicacion.setAttribute('DistanciaRecorrida',
+                                                      str(ubicacion_data.get('DistanciaRecorrida')))
                     cartaporte_ubicacion.setAttribute('TipoUbicacion', 'Destino')
                     cartaporte_ubicacion.setAttribute('RFCRemitenteDestinatario', ubicacion_data.get('RFCDestinatario'))
                     # cartaporte_ubicacion.setAttribute('NombreRFC', ubicacion_data.get('NombreDestinatario'))
-                    cartaporte_ubicacion.setAttribute('FechaHoraSalidaLlegada', ubicacion_data.get('FechaHoraProgLlegada'))
+                    cartaporte_ubicacion.setAttribute('FechaHoraSalidaLlegada',
+                                                      ubicacion_data.get('FechaHoraProgLlegada'))
                     # cartaporte_ubicacion_destino = self.dom.createElement('cartaporte20:Destino')
                     # cartaporte_ubicacion_destino.setAttribute('RFCDestinatario', ubicacion_data.get('RFCDestinatario'))
                     # cartaporte_ubicacion_destino.setAttribute('NombreDestinatario', ubicacion_data.get('NombreDestinatario'))
                     # cartaporte_ubicacion_destino.setAttribute('FechaHoraProgLlegada', ubicacion_data.get('FechaHoraProgLlegada'))
-                    #cartaporte_ubicacion.appendChild(cartaporte_ubicacion_destino)
+                    # cartaporte_ubicacion.appendChild(cartaporte_ubicacion_destino)
                     total_distancia_recorrida += float(ubicacion_data.get('DistanciaRecorrida'))
                     destination_counter += 1
 
@@ -493,7 +496,8 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
 
                 if mercancia_data.get('MaterialPeligroso') == 'Si':
                     cartaporte_mercancia.setAttribute('MaterialPeligroso', mercancia_data.get('MaterialPeligroso'))
-                    cartaporte_mercancia.setAttribute('CveMaterialPeligroso', mercancia_data.get('CveMaterialPeligroso'))
+                    cartaporte_mercancia.setAttribute('CveMaterialPeligroso',
+                                                      mercancia_data.get('CveMaterialPeligroso'))
                     cartaporte_mercancia.setAttribute('Embalaje', mercancia_data.get('Embalaje'))
                     cartaporte_mercancia.setAttribute('DescripEmbalaje', mercancia_data.get('DescripEmbalaje'))
 
@@ -512,7 +516,8 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
             cartaporte_mercancias.setAttribute('UnidadPeso', 'KGM')
             cartaporte.appendChild(cartaporte_mercancias)
 
-            if self.tipo_transporte not in ['AutotransporteFederal', 'TransporteMaritimo', 'TransporteAereo', 'TransporteFerroviario']:
+            if self.tipo_transporte not in ['AutotransporteFederal', 'TransporteMaritimo', 'TransporteAereo',
+                                            'TransporteFerroviario']:
                 raise Exception('Se debe registrar un tipo de transporte')
             transporte_data = self.data.get('transporte')
             if self.tipo_transporte == 'AutotransporteFederal':
@@ -523,12 +528,12 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 # cartaporte_tipo_transporte.setAttribute('NumPolizaSeguro', transporte_data.get('NumPolizaSeguro'))
 
                 cartaporte_identificacion_vehicular = self.dom.createElement('cartaporte20:IdentificacionVehicular')
-                cartaporte_identificacion_vehicular.setAttribute('ConfigVehicular', transporte_data.get('ConfigVehicular'))
+                cartaporte_identificacion_vehicular.setAttribute('ConfigVehicular',
+                                                                 transporte_data.get('ConfigVehicular'))
                 cartaporte_identificacion_vehicular.setAttribute('PlacaVM', transporte_data.get('PlacaVM'))
                 cartaporte_identificacion_vehicular.setAttribute('AnioModeloVM', transporte_data.get('AnioModeloVM'))
                 cartaporte_tipo_transporte.appendChild(cartaporte_identificacion_vehicular)
 
-
                 cartaporte_seguros = self.dom.createElement('cartaporte20:Seguros')
                 cartaporte_seguros.setAttribute('AseguraRespCivil', transporte_data.get('NombreAseg'))
                 cartaporte_seguros.setAttribute('PolizaRespCivil', transporte_data.get('NumPolizaSeguro'))
